# Copyright (c) 2017 Ericsson.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import yaml

from tempest.lib import decorators

from heat_tempest_plugin.common import test
from heat_tempest_plugin.tests.functional import functional_base


test_template = '''
heat_template_version: pike
description: Test template to create, update, delete trunk.
resources:
  parent_net:
    type: OS::Neutron::Net
  trunk_net_one:
    type: OS::Neutron::Net
  trunk_net_two:
    type: OS::Neutron::Net
  parent_subnet:
    type: OS::Neutron::Subnet
    properties:
      network: { get_resource: parent_net }
      cidr: 10.0.0.0/16
  trunk_subnet_one:
    type: OS::Neutron::Subnet
    properties:
      network: { get_resource: trunk_net_one }
      cidr: 10.10.0.0/16
  trunk_subnet_two:
    type: OS::Neutron::Subnet
    properties:
      network: { get_resource: trunk_net_two }
      cidr: 10.20.0.0/16
  parent_port:
    type: OS::Neutron::Port
    properties:
      network: { get_resource: parent_net }
      name: trunk_parent_port
  sub_port_one:
    type: OS::Neutron::Port
    properties:
      network: { get_resource: trunk_net_one }
      name: trunk_sub_port_one
  sub_port_two:
    type: OS::Neutron::Port
    properties:
      network: { get_resource: trunk_net_two }
      name: trunk_sub_port_two
  trunk:
    type: OS::Neutron::Trunk
    properties:
      name: test_trunk
      port: { get_resource: parent_port }
      sub_ports:
outputs:
  trunk_parent_port:
    value: { get_attr: [trunk, port_id] }
'''


@test.requires_resource_type('OS::Neutron::Trunk')
class UpdateTrunkTest(functional_base.FunctionalTestsBase):

    @staticmethod
    def _sub_ports_dict_to_set(sub_ports):
        new_sub_ports = copy.deepcopy(sub_ports)

        # NOTE(lajos katona): In the template we have to give the sub port as
        # port, but from trunk_details we receive back them with port_id.
        # As an extra trunk_details contains the mac_address as well which is
        # useless here.
        # So here we have to make sure that the dictionary (input from
        # template or output from trunk_details) have the same keys:
        if any('mac_address' in d for d in new_sub_ports):
            for sp in new_sub_ports:
                sp['port'] = sp['port_id']
                del sp['port_id']
                del sp['mac_address']

        # NOTE(lajos katona): We receive lists (trunk_details['sub_ports'] and
        # the input to the template) and we can't be sure that the order is the
        # same, so by using sets we can compare them.
        sub_ports_set = {frozenset(d.items()) for d in new_sub_ports}
        return sub_ports_set

    @decorators.idempotent_id('5572b0ac-fdb2-4c68-a49e-024771814471')
    def test_add_first_sub_port(self):
        stack_identifier = self.stack_create(template=test_template)

        parsed_template = yaml.safe_load(test_template)
        new_sub_port = [{'port': {'get_resource': 'sub_port_one'},
                         'segmentation_id': 10,
                         'segmentation_type': 'vlan'}]
        parsed_template['resources']['trunk']['properties'][
            'sub_ports'] = new_sub_port
        updated_template = yaml.safe_dump(parsed_template)
        self.update_stack(stack_identifier, updated_template)

        # Fix the port_id in the template for assertion
        new_sub_port[0]['port'] = self.get_physical_resource_id(
            stack_identifier, 'sub_port_one')
        parent_id = self.get_stack_output(
            stack_identifier, 'trunk_parent_port')
        parent_port = self.network_client.show_port(parent_id)['port']
        trunk_sub_port = parent_port['trunk_details']['sub_ports']

        self.assertEqual(self._sub_ports_dict_to_set(new_sub_port),
                         self._sub_ports_dict_to_set(trunk_sub_port))

    @decorators.idempotent_id('c3f52330-01b7-4649-99fd-43700e6bbda3')
    def test_add_a_second_sub_port(self):
        parsed_template = yaml.safe_load(test_template)
        sub_ports = [{'port': {'get_resource': 'sub_port_one'},
                      'segmentation_type': 'vlan',
                      'segmentation_id': 10}, ]
        parsed_template['resources']['trunk']['properties'][
            'sub_ports'] = sub_ports
        template_with_sub_ports = yaml.safe_dump(parsed_template)

        stack_identifier = self.stack_create(template=template_with_sub_ports)

        new_sub_port = {'port': {'get_resource': 'sub_port_two'},
                        'segmentation_id': 20,
                        'segmentation_type': 'vlan'}
        parsed_template['resources']['trunk']['properties'][
            'sub_ports'].append(new_sub_port)

        updated_template = yaml.safe_dump(parsed_template)

        self.update_stack(stack_identifier, updated_template)

        # Fix the port_ids in the templates for assertion
        sub_ports[0]['port'] = self.get_physical_resource_id(
            stack_identifier, 'sub_port_one')
        new_sub_port['port'] = self.get_physical_resource_id(
            stack_identifier, 'sub_port_two')
        expected_sub_ports = [sub_ports[0], new_sub_port]

        parent_id = self.get_stack_output(
            stack_identifier, 'trunk_parent_port')
        parent_port = self.network_client.show_port(parent_id)['port']
        trunk_sub_ports = parent_port['trunk_details']['sub_ports']

        self.assertEqual(self._sub_ports_dict_to_set(expected_sub_ports),
                         self._sub_ports_dict_to_set(trunk_sub_ports))

    @decorators.idempotent_id('cb59363e-5517-42f9-8b93-9ad700e2ef4c')
    def test_remove_sub_port_from_trunk(self):
        sub_ports = [{'port': {'get_resource': 'sub_port_one'},
                      'segmentation_type': 'vlan',
                      'segmentation_id': 10},
                     {'port': {'get_resource': 'sub_port_two'},
                      'segmentation_type': 'vlan',
                      'segmentation_id': 20}]
        parsed_template = yaml.safe_load(test_template)
        parsed_template['resources']['trunk']['properties'][
            'sub_ports'] = sub_ports
        template_with_sub_ports = yaml.safe_dump(parsed_template)

        stack_identifier = self.stack_create(template=template_with_sub_ports)

        sub_port_to_be_removed = {'port': {'get_resource': 'sub_port_two'},
                                  'segmentation_type': 'vlan',
                                  'segmentation_id': 20}
        parsed_template['resources']['trunk'][
            'properties']['sub_ports'].remove(sub_port_to_be_removed)
        updated_template = yaml.safe_dump(parsed_template)

        self.update_stack(stack_identifier, updated_template)

        # Fix the port_ids in the templates for assertion
        sub_ports[0]['port'] = self.get_physical_resource_id(
            stack_identifier, 'sub_port_one')
        expected_sub_ports = [sub_ports[0]]

        parent_id = self.get_stack_output(
            stack_identifier, 'trunk_parent_port')
        parent_port = self.network_client.show_port(parent_id)['port']
        trunk_sub_ports = parent_port['trunk_details']['sub_ports']

        self.assertEqual(self._sub_ports_dict_to_set(expected_sub_ports),
                         self._sub_ports_dict_to_set(trunk_sub_ports))

    @decorators.idempotent_id('7f0836c7-1d35-4d11-bcdc-e0e19ca68b68')
    def test_remove_last_sub_port_from_trunk(self):
        sub_ports = [{'port': {'get_resource': 'sub_port_one'},
                      'segmentation_type': 'vlan',
                      'segmentation_id': 10}]
        parsed_template = yaml.safe_load(test_template)
        parsed_template['resources']['trunk']['properties'][
            'sub_ports'] = sub_ports

        template_with_sub_ports = yaml.safe_dump(parsed_template)
        stack_identifier = self.stack_create(template=template_with_sub_ports)

        sub_port_to_be_removed = {'port': {'get_resource': 'sub_port_one'},
                                  'segmentation_type': 'vlan',
                                  'segmentation_id': 10}

        parsed_template['resources']['trunk'][
            'properties']['sub_ports'] = []
        updated_template = yaml.safe_dump(parsed_template)

        self.update_stack(stack_identifier, updated_template)

        sub_port_to_be_removed['port'] = self.get_physical_resource_id(
            stack_identifier, 'sub_port_one')
        parent_id = self.get_stack_output(
            stack_identifier, 'trunk_parent_port')
        parent_port = self.network_client.show_port(parent_id)['port']
        trunk_sub_ports = parent_port['trunk_details']['sub_ports']

        self.assertNotEqual(
            self._sub_ports_dict_to_set([sub_port_to_be_removed]),
            self._sub_ports_dict_to_set(trunk_sub_ports))
        self.assertFalse(trunk_sub_ports,
                         'The returned sub ports (%s) in trunk_details is '
                         'not empty!' % trunk_sub_ports)

    @decorators.idempotent_id('e9296d6f-009c-4530-9aaf-84a0f8281bcb')
    def test_update_existing_sub_port_on_trunk(self):
        sub_ports = [{'port': {'get_resource': 'sub_port_one'},
                      'segmentation_type': 'vlan',
                      'segmentation_id': 10}]
        parsed_template = yaml.safe_load(test_template)
        parsed_template['resources']['trunk']['properties'][
            'sub_ports'] = sub_ports

        template_with_sub_ports = yaml.safe_dump(parsed_template)
        stack_identifier = self.stack_create(template=template_with_sub_ports)

        sub_port_id = self.get_physical_resource_id(
            stack_identifier, 'sub_port_one')
        parsed_template['resources']['trunk']['properties']['sub_ports'][0][
            'segmentation_id'] = 99
        updated_template = yaml.safe_dump(parsed_template)

        self.update_stack(stack_identifier, updated_template)
        updated_sub_port = {'port': sub_port_id,
                            'segmentation_type': 'vlan',
                            'segmentation_id': 99}
        parent_id = self.get_stack_output(
            stack_identifier, 'trunk_parent_port')
        parent_port = self.network_client.show_port(parent_id)['port']
        trunk_sub_ports = parent_port['trunk_details']['sub_ports']

        self.assertEqual(self._sub_ports_dict_to_set([updated_sub_port]),
                         self._sub_ports_dict_to_set(trunk_sub_ports))

    @decorators.idempotent_id('912044d8-e0c1-4c6c-ab4e-bc1cadc46c18')
    def test_update_trunk_name_and_description(self):
        new_name = 'pineapple'
        new_description = 'This is a test trunk'

        stack_identifier = self.stack_create(template=test_template)
        parsed_template = yaml.safe_load(test_template)
        parsed_template['resources']['trunk']['properties']['name'] = new_name
        parsed_template['resources']['trunk']['properties'][
            'description'] = new_description
        updated_template = yaml.safe_dump(parsed_template)
        self.update_stack(stack_identifier, template=updated_template)

        parent_id = self.get_stack_output(
            stack_identifier, 'trunk_parent_port')
        parent_port = self.network_client.show_port(parent_id)['port']
        trunk_id = parent_port['trunk_details']['trunk_id']

        trunk = self.network_client.show_trunk(trunk_id)['trunk']
        self.assertEqual(new_name, trunk['name'])
        self.assertEqual(new_description, trunk['description'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heatclient.common import template_utils
import six
from tempest.lib import decorators

from heat_tempest_plugin.tests.scenario import scenario_base

CFG1_SH = '''#!/bin/sh
echo "Writing to /tmp/$bar"
echo $foo > /tmp/$bar
echo -n "The file /tmp/$bar contains `cat /tmp/$bar` for server \
$deploy_server_id during $deploy_action" > $heat_outputs_path.result
echo "Written to /tmp/$bar"
echo "Output to stderr" 1>&2
'''

CFG3_PP = '''file {'barfile':
  ensure  => file,
  mode    => '0644',
  path    => "/tmp/$::bar",
  content => "$::foo",
}
file {'output_result':
  ensure  => file,
  path    => "$::heat_outputs_path.result",
  mode    => '0644',
  content => "The file /tmp/$::bar contains $::foo for server \
$::deploy_server_id during $::deploy_action",
}
'''


class SoftwareConfigIntegrationTest(scenario_base.ScenarioTestsBase):

    def setUp(self):
        super(SoftwareConfigIntegrationTest, self).setUp()
        if not self.conf.image_ref:
            raise self.skipException("No image configured to test")
        if not self.conf.instance_type:
            raise self.skipException("No flavor configured to test")

    def check_stack(self):
        sid = self.stack_identifier
        # Check that all stack resources were created
        for res in ('cfg2a', 'cfg2b', 'cfg1', 'cfg3', 'server'):
            self._wait_for_resource_status(
                sid, res, 'CREATE_COMPLETE')

        server_resource = self.client.resources.get(sid, 'server')
        server_id = server_resource.physical_resource_id
        server = self.compute_client.servers.get(server_id)

        # Waiting for each deployment to contribute their
        # config to resource
        try:
            for res in ('dep2b', 'dep1', 'dep3'):
                self._wait_for_resource_status(
                    sid, res, 'CREATE_IN_PROGRESS')

            server_metadata = self.client.resources.metadata(
                sid, 'server')
            deployments = dict((d['name'], d) for d in
                               server_metadata['deployments'])

            for res in ('dep2a', 'dep2b', 'dep1', 'dep3'):
                self._wait_for_resource_status(
                    sid, res, 'CREATE_COMPLETE')
        finally:
            # attempt to log the server console regardless of deployments
            # going to complete. This allows successful and failed boot
            # logs to be compared
            self._log_console_output(servers=[server])

        complete_server_metadata = self.client.resources.metadata(
            sid, 'server')

        # Ensure any previously available deployments haven't changed so
        # config isn't re-triggered
        complete_deployments = dict((d['name'], d) for d in
                                    complete_server_metadata['deployments'])
        for k, v in six.iteritems(deployments):
            self.assertEqual(v, complete_deployments[k])

        stack = self.client.stacks.get(sid)

        res1 = self._stack_output(stack, 'res1')
        self.assertEqual(
            'The file %s contains %s for server %s during %s' % (
                '/tmp/baaaaa', 'fooooo', server_id, 'CREATE'),
            res1['result'])
        self.assertEqual(0, res1['status_code'])
        self.assertEqual('Output to stderr\n', res1['stderr'])
        self.assertGreater(len(res1['stdout']), 0)

        res2 = self._stack_output(stack, 'res2')
        self.assertEqual(
            'The file %s contains %s for server %s during %s' % (
                '/tmp/cfn-init-foo', 'barrr', server_id, 'CREATE'),
            res2['result'])
        self.assertEqual(0, res2['status_code'])
        self.assertEqual('', res2['stderr'])
        self.assertEqual('', res2['stdout'])

        res3 = self._stack_output(stack, 'res3')
        self.assertEqual(
            'The file %s contains %s for server %s during %s' % (
                '/tmp/ba', 'fo', server_id, 'CREATE'),
            res3['result'])
        self.assertEqual(0, res3['status_code'])
        self.assertEqual('', res3['stderr'])
        self.assertGreater(len(res1['stdout']), 0)

        dep1_resource = self.client.resources.get(sid, 'dep1')
        dep1_id = dep1_resource.physical_resource_id
        dep1_dep = self.client.software_deployments.get(dep1_id)
        if hasattr(dep1_dep, 'updated_time'):
            # Only check updated_time if the attribute exists.
            # This allows latest heat agent code to be tested with
            # Juno heat (which doesn't expose updated_time)
            self.assertIsNotNone(dep1_dep.updated_time)
            self.assertNotEqual(
                dep1_dep.updated_time,
                dep1_dep.creation_time)

    @decorators.idempotent_id('fb2afe23-d1a8-45fc-bf8d-b18fc8412972')
    def test_server_software_config(self):
        """Check that passed files with scripts are executed on created server.

        The alternative scenario is the following:
            1. Create a stack and pass files with scripts.
            2. Check that all stack resources are created successfully.
            3. Wait for all deployments.
            4. Check that stack was created.
            5. Check stack outputs.
        """

        parameters = {
            'key_name': self.keypair_name,
            'flavor': self.conf.instance_type,
            'image': self.conf.image_ref,
            'network': self.net['id']
        }

        files = {
            'file:///cfg1.sh': CFG1_SH,
            'file:///cfg3.pp': CFG3_PP
        }

        env_files, env = template_utils.process_environment_and_files(
            self.conf.boot_config_env)

        # Launch stack
        self.stack_identifier = self.launch_stack(
            template_name='test_server_software_config.yaml',
            parameters=parameters,
            files=dict(list(files.items()) + list(env_files.items())),
            expected_status=None,
            environment=env
        )

        # Check stack
        self.check_stack()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


server_with_sub_fixed_ip_template = '''
heat_template_version: 2016-04-08
description: Test template to test nova server with subnet and fixed_ip.
parameters:
  flavor:
    type: string
  image:
    type: string
resources:
  net:
    type: OS::Neutron::Net
    properties:
      name: my_net
  subnet:
    type: OS::Neutron::Subnet
    properties:
      network: {get_resource: net}
      cidr: 11.11.11.0/24
  security_group:
    type: OS::Neutron::SecurityGroup
    properties:
      name: the_sg
      description: Ping and SSH
      rules:
      - protocol: icmp
      - protocol: tcp
        port_range_min: 22
        port_range_max: 22
  server:
    type: OS::Nova::Server
    properties:
      image: {get_param: image}
      flavor: {get_param: flavor}
      networks:
        - subnet: {get_resource: subnet}
          fixed_ip: 11.11.11.11
      security_groups:
        - {get_resource: security_group}
outputs:
  networks:
    value: {get_attr: [server, networks]}
'''

server_with_port_template = '''
heat_template_version: 2016-04-08
description: Test template to test nova server with port.
parameters:
  flavor:
    type: string
  image:
    type: string
resources:
  net:
    type: OS::Neutron::Net
    properties:
      name: server_with_port_net
  subnet:
    type: OS::Neutron::Subnet
    properties:
      network: {get_resource: net}
      cidr: 11.11.11.0/24
  port:
    type: OS::Neutron::Port
    properties:
      network: {get_resource: net}
      fixed_ips:
        - subnet: {get_resource: subnet}
          ip_address: 11.11.11.11
  server:
    type: OS::Nova::Server
    properties:
      image: {get_param: image}
      flavor: {get_param: flavor}
      networks:
        - port: {get_resource: port}
'''

server_with_multiple_subnets_no_ports_template = '''
heat_template_version: 2016-04-08
description: Test template to test nova server network updates.
parameters:
  flavor:
    type: string
  image:
    type: string
resources:
  net:
    type: OS::Neutron::Net
    properties:
      name: the_net
  subnet_a:
    type: OS::Neutron::Subnet
    properties:
      network: {get_resource: net}
      cidr: 11.11.11.0/24
      name: subnet_a
  subnet_b:
    type: OS::Neutron::Subnet
    properties:
      network: {get_resource: net}
      cidr: 12.12.12.0/24
      name: subnet_b
  server:
    type: OS::Nova::Server
    properties:
      image: {get_param: image}
      flavor: {get_param: flavor}
      networks: $NETWORKS
outputs:
  networks:
    value: {get_attr: [server, networks]}
'''

server_with_no_nets_template = '''
heat_template_version: 2016-04-08
description: Test template to test nova server network updates.
parameters:
  flavor:
    type: string
  image:
    type: string
resources:
  net:
    type: OS::Neutron::Net
    properties:
      name: the_net
  subnet:
    type: OS::Neutron::Subnet
    properties:
      name: the_subnet
      network: {get_resource: net}
      cidr: 11.11.11.0/24
  server:
    type: OS::Nova::Server
    properties:
      image: {get_param: image}
      flavor: {get_param: flavor}
      networks: $NETWORKS
outputs:
  port0_id:
    value: {get_attr: [server, addresses, the_net, 0, port]}
  port1_id:
    value: {get_attr: [server, addresses, the_net, 1, port]}
  port2_id:
    value: {get_attr: [server, addresses, the_net, 2, port]}
  port0_ip_addr:
    value: {get_attr: [server, addresses, the_net, 0, addr]}
  port1_ip_addr:
    value: {get_attr: [server, addresses, the_net, 1, addr]}
  port2_ip_addr:
    value: {get_attr: [server, addresses, the_net, 2, addr]}
'''


class CreateServerTest(functional_base.FunctionalTestsBase):

    def get_outputs(self, stack_identifier, output_key):
        stack = self.client.stacks.get(stack_identifier)
        return self._stack_output(stack, output_key)

    @decorators.idempotent_id('58ccf0aa-7531-4eaa-8ed5-38663a4defaa')
    def test_create_server_with_subnet_fixed_ip_sec_group(self):
        parms = {'flavor': self.conf.minimal_instance_type,
                 'image': self.conf.minimal_image_ref}
        stack_identifier = self.stack_create(
            template=server_with_sub_fixed_ip_template,
            stack_name='server_with_sub_ip',
            parameters=parms)

        networks = self.get_outputs(stack_identifier, 'networks')
        self.assertEqual(['11.11.11.11'], networks['my_net'])

        server_resource = self.client.resources.get(
            stack_identifier, 'server')
        server_id = server_resource.physical_resource_id
        server = self.compute_client.servers.get(server_id)
        self.assertEqual([{"name": "the_sg"}], server.security_groups)

    @decorators.idempotent_id('12185eaa-927f-43f3-a525-0424c4eb9b5d')
    def test_create_update_server_with_subnet(self):
        parms = {'flavor': self.conf.minimal_instance_type,
                 'image': self.conf.minimal_image_ref}
        template = server_with_multiple_subnets_no_ports_template.replace(
            '$NETWORKS', ('[{subnet: {get_resource: subnet_a}}]'))
        stack_identifier = self.stack_create(
            template=template,
            stack_name='create_server_with_sub_ip',
            parameters=parms)
        networks = self.get_outputs(stack_identifier, 'networks')
        self.assertIn('11.11.11', networks['the_net'][0])

        # update the server using a different subnet, we won't pass
        # both port_id and net_id to attach interface, then update success
        template = server_with_multiple_subnets_no_ports_template.replace(
            '$NETWORKS', ('[{subnet: {get_resource: subnet_b}}]'))
        self.update_stack(stack_identifier,
                          template,
                          parameters=parms)
        new_networks = self.get_outputs(stack_identifier, 'networks')
        self.assertNotIn('11.11.11', new_networks['the_net'][0])
        self.assertIn('12.12.12', new_networks['the_net'][0])

    @decorators.idempotent_id('19479c15-6b25-4865-8889-658566608bd9')
    def test_create_server_with_port(self):
        parms = {'flavor': self.conf.minimal_instance_type,
                 'image': self.conf.minimal_image_ref}
        # We just want to make sure we can create the server, no need to assert
        # anything
        self.stack_create(
            template=server_with_port_template,
            stack_name='server_with_port',
            parameters=parms)


class UpdateServerNetworksTest(functional_base.FunctionalTestsBase):
    def setUp(self):
        super(UpdateServerNetworksTest, self).setUp()
        self.params = {'flavor': self.conf.minimal_instance_type,
                       'image': self.conf.minimal_image_ref}

    def get_outputs(self, stack_identifier, output_key):
        stack = self.client.stacks.get(stack_identifier)
        return self._stack_output(stack, output_key)

    @decorators.idempotent_id('c1a22dbf-3160-41b7-8d3f-62ca33fc35a8')
    def test_create_update_server_swap_network_subnet(self):
        '''Test updating stack with:

        old_snippet
          networks:
            - network: {get_resource: net}
        new_snippet
          networks:
            - subnet: {get_resource: subnet}
        '''
        template = server_with_no_nets_template.replace(
            '$NETWORKS', '[{network: {get_resource: net}}]')
        stack_identifier = self.stack_create(
            template=template,
            stack_name='swap_network_subnet',
            parameters=self.params)
        port0 = self.get_outputs(stack_identifier, 'port0_id')
        template_update = server_with_no_nets_template.replace(
            '$NETWORKS', '[{subnet: {get_resource: subnet}}]')
        self.update_stack(stack_identifier, template_update,
                          parameters=self.params)
        self.assertEqual(port0, self.get_outputs(stack_identifier, 'port0_id'))

    @decorators.idempotent_id('cccfe612-1ab7-401f-a4c5-63372826a780')
    def test_create_update_server_swap_network_port(self):
        '''Test updating stack with:

        old_snippet
          networks:
            - network: {get_resource: net}
        new_snippet
          networks:
            - port: <the_port_created_on_stack_create>
        '''
        template = server_with_no_nets_template.replace(
            '$NETWORKS', '[{network: {get_resource: net}}]')
        stack_identifier = self.stack_create(
            template=template,
            stack_name='swap_network_port',
            parameters=self.params)
        port0 = self.get_outputs(stack_identifier, 'port0_id')
        template_update = server_with_no_nets_template.replace(
            '$NETWORKS', '[{port: ' + port0 + '}]')
        self.update_stack(stack_identifier, template_update,
                          parameters=self.params)
        self.assertEqual(port0, self.get_outputs(stack_identifier, 'port0_id'))

    @decorators.idempotent_id('3eeb0dff-5d2d-4178-a4e6-06e4c26ce23a')
    def test_create_update_server_swap_subnet_network(self):
        '''Test updating stack with:

        old_snippet
          networks:
            - subnet: {get_resource: subnet}
        new_snippet
          networks:
            - network: {get_resource: net}
        '''
        template = server_with_no_nets_template.replace(
            '$NETWORKS', '[{subnet: {get_resource: subnet}}]')
        stack_identifier = self.stack_create(
            template=template,
            stack_name='swap_subnet_network',
            parameters=self.params)
        port0 = self.get_outputs(stack_identifier, 'port0_id')
        template_update = server_with_no_nets_template.replace(
            '$NETWORKS', '[{network: {get_resource: net}}]')
        self.update_stack(stack_identifier, template_update,
                          parameters=self.params)
        self.assertEqual(port0, self.get_outputs(stack_identifier, 'port0_id'))

    @decorators.idempotent_id('647fda5d-fc0c-4eb1-9ce3-c4c537461324')
    def test_create_update_server_add_subnet(self):
        '''Test updating stack with:

        old_snippet
          networks:
            - network: {get_resource: net}
        new_snippet
          networks:
            - network: {get_resource: net}
              subnet: {get_resource: subnet}
        '''
        template = server_with_no_nets_template.replace(
            '$NETWORKS', '[{network: {get_resource: net}}]')
        stack_identifier = self.stack_create(
            template=template,
            stack_name='add_subnet',
            parameters=self.params)
        port0 = self.get_outputs(stack_identifier, 'port0_id')
        template_update = server_with_no_nets_template.replace(
            '$NETWORKS',
            '[{network: {get_resource: net}, subnet: {get_resource: subnet}}]')
        self.update_stack(stack_identifier, template_update,
                          parameters=self.params)
        self.assertEqual(port0, self.get_outputs(stack_identifier, 'port0_id'))

    @decorators.idempotent_id('01c0f1cd-25b2-49b9-b4ac-fc4dd8937e42')
    def test_create_update_server_add_same_fixed_ip(self):
        '''Test updating stack with:

        old_snippet
          networks:
            - network: {get_resource: net}
        new_snippet
          networks:
            - network: {get_resource: net}
              fixed_ip: <the_same_ip_already_allocated>
        '''
        template = server_with_no_nets_template.replace(
            '$NETWORKS',
            '[{network: {get_resource: net}}]')
        stack_identifier = self.stack_create(
            template=template,
            stack_name='same_fixed_ip',
            parameters=self.params)
        port0 = self.get_outputs(stack_identifier, 'port0_id')
        port0_ip = self.get_outputs(stack_identifier, 'port0_ip_addr')
        template_update = server_with_no_nets_template.replace(
            '$NETWORKS',
            '[{network: {get_resource: net}, fixed_ip: ' + port0_ip + '}]')
        self.update_stack(stack_identifier, template_update,
                          parameters=self.params)
        self.assertEqual(port0, self.get_outputs(stack_identifier, 'port0_id'))

    @decorators.idempotent_id('abc39cd6-7745-4314-ac04-85df532dd7c9')
    def test_create_update_server_add_network(self):
        '''Test updating stack with:

        old_snippet
          networks:
            - subnet: {get_resource: subnet}
        new_snippet
          networks:
            - network: {get_resource: net}
              subnet: {get_resource: subnet}
        '''
        template = server_with_no_nets_template.replace(
            '$NETWORKS', '[{subnet: {get_resource: subnet}}]')
        stack_identifier = self.stack_create(
            template=template,
            stack_name='add_network',
            parameters=self.params)
        port0 = self.get_outputs(stack_identifier, 'port0_id')
        template_update = server_with_no_nets_template.replace(
            '$NETWORKS',
            '[{network: {get_resource: net}, subnet: {get_resource: subnet}}]')
        self.update_stack(stack_identifier, template_update,
                          parameters=self.params)
        self.assertEqual(port0, self.get_outputs(stack_identifier, 'port0_id'))

    @decorators.idempotent_id('3f729e7e-a698-4ee3-8a5e-0db84f16d1e1')
    def test_create_update_server_multi_networks_swaps(self):
        '''Test updating stack with:

         old_snippet:
           networks:
             - network: {get_resource: net}
             - network: {get_resource: net}
               fixed_ip: 11.11.11.33
             - subnet: {get_resource: subnet}
         new_snippet:
           networks:
             - subnet: {get_resource: subnet}
             - network: {get_resource: net}
             - network: {get_resource: net}
               subnet: {get_resource: subnet}
        '''
        old_snippet = """
        - network: {get_resource: net}
        - network: {get_resource: net}
          fixed_ip: 11.11.11.33
        - subnet: {get_resource: subnet}
"""
        new_snippet = """
        - subnet: {get_resource: subnet}
        - network: {get_resource: net}
        - network: {get_resource: net}
          subnet: {get_resource: subnet}
"""
        template = server_with_no_nets_template.replace(
            '$NETWORKS', old_snippet)
        stack_identifier = self.stack_create(
            template=template,
            stack_name='multi_networks_swaps',
            parameters=self.params)
        port0 = self.get_outputs(stack_identifier, 'port0_id')
        port1 = self.get_outputs(stack_identifier, 'port1_id')
        port2 = self.get_outputs(stack_identifier, 'port2_id')
        template_update = server_with_no_nets_template.replace(
            '$NETWORKS', new_snippet)
        self.update_stack(stack_identifier, template_update,
                          parameters=self.params)
        self.assertEqual(port0, self.get_outputs(stack_identifier, 'port0_id'))
        self.assertEqual(port1, self.get_outputs(stack_identifier, 'port1_id'))
        self.assertEqual(port2, self.get_outputs(stack_identifier, 'port2_id'))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from keystoneclient.v3 import client as keystoneclient
from tempest.lib import decorators
from zaqarclient.queues.v2 import client as zaqarclient

from heat_tempest_plugin.common import test
from heat_tempest_plugin.tests.functional import functional_base


@test.requires_service_type('messaging')
class ZaqarWaitConditionTest(functional_base.FunctionalTestsBase):
    template = '''
heat_template_version: "2013-05-23"

resources:
  wait_condition:
    type: OS::Heat::WaitCondition
    properties:
      handle: {get_resource: wait_handle}
      timeout: 120
  wait_handle:
    type: OS::Heat::WaitConditionHandle
    properties:
      signal_transport: ZAQAR_SIGNAL

outputs:
  wait_data:
   value: {'Fn::Select': ['data_id', {get_attr: [wait_condition, data]}]}
'''

    @decorators.idempotent_id('90183f0d-9929-43a6-8fb6-b81003824c6d')
    def test_signal_queues(self):
        stack_identifier = self.stack_create(
            template=self.template,
            expected_status=None)
        self._wait_for_resource_status(stack_identifier, 'wait_handle',
                                       'CREATE_COMPLETE')
        resource = self.client.resources.get(stack_identifier, 'wait_handle')
        signal = json.loads(resource.attributes['signal'])
        ks = keystoneclient.Client(
            auth_url=signal['auth_url'],
            user_id=signal['user_id'],
            password=signal['password'],
            project_id=signal['project_id'],
            insecure=self.conf.disable_ssl_certificate_validation,
            cacert=self.conf.ca_file)
        endpoint = ks.service_catalog.url_for(
            service_type='messaging', endpoint_type=self.conf.endpoint_type)
        conf = {
            'auth_opts': {
                'backend': 'keystone',
                'options': {
                    'os_auth_token': ks.auth_token,
                    'os_project_id': signal['project_id']
                }
            }
        }

        zaqar = zaqarclient.Client(endpoint, conf=conf)

        queue = zaqar.queue(signal['queue_id'])
        queue.post({'body': {'data': 'here!', 'id': 'data_id'}, 'ttl': 600})
        self._wait_for_stack_status(stack_identifier, 'CREATE_COMPLETE')
        stack = self.client.stacks.get(stack_identifier)
        self.assertEqual('here!', stack.outputs[0]['output_value'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heatclient.common import template_utils
from tempest.lib import decorators

from heat_tempest_plugin.common import test
from heat_tempest_plugin.tests.scenario import scenario_base


class BasicResourcesTest(scenario_base.ScenarioTestsBase):

    def setUp(self):
        super(BasicResourcesTest, self).setUp()
        if not self.conf.image_ref:
            raise self.skipException("No image configured to test")
        if not self.conf.instance_type:
            raise self.skipException("No flavor configured to test")

    def check_stack(self):
        sid = self.stack_identifier
        # Check that stack were created
        self._wait_for_stack_status(sid, 'CREATE_COMPLETE')
        server_resource = self.client.resources.get(sid, 'server')
        server_id = server_resource.physical_resource_id
        server = self.compute_client.servers.get(server_id)
        self.assertEqual(server.id, server_id)

        stack = self.client.stacks.get(sid)

        server_networks = self._stack_output(stack, 'server_networks')
        self.assertIn(self.private_net_name, server_networks)

    @decorators.idempotent_id('bd151ea4-2dcd-4440-9bf0-eff63a98e5d4')
    def test_base_resources_integration(self):
        """Define test for base resources interation from core porjects

        The alternative scenario is the following:
            1. Create a stack with basic resources from core projects.
            2. Check that all stack resources are created successfully.
            3. Wait for deployment.
            4. Check that stack was created.
            5. Check stack outputs.
        """

        self.private_net_name = test.rand_name('heat-net')
        parameters = {
            'key_name': test.rand_name('heat-key'),
            'flavor': self.conf.instance_type,
            'image': self.conf.image_ref,
            'vol_size': self.conf.volume_size,
            'private_net_name': self.private_net_name
        }

        env_files, env = template_utils.process_environment_and_files(
            self.conf.boot_config_env)

        # Launch stack
        self.stack_identifier = self.launch_stack(
            template_name='test_base_resources.yaml',
            parameters=parameters,
            expected_status=None,
            environment=env
        )

        # Check stack
        self.check_stack()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import uuid

from tempest.lib import decorators
from zaqarclient.queues.v2 import client as zaqarclient

from heat_tempest_plugin.common import test
from heat_tempest_plugin.tests.functional import functional_base


@test.requires_service_type('messaging')
class ZaqarEventSinkTest(functional_base.FunctionalTestsBase):
    template = '''
heat_template_version: "2013-05-23"
resources:
  test_resource:
    type: OS::Heat::TestResource
    properties:
      value: ok
'''

    @decorators.idempotent_id('d0b72695-e97d-4aa1-bfaf-31c14b09ac87')
    def test_events(self):
        queue_id = str(uuid.uuid4())
        environment = {'event_sinks': [{'type': 'zaqar-queue',
                                        'target': queue_id,
                                        'ttl': 120}]}
        stack_identifier = self.stack_create(
            template=self.template,
            environment=environment)
        stack_name, stack_id = stack_identifier.split('/')
        conf = {
            'auth_opts': {
                'backend': 'keystone',
                'options': {
                    'os_username': self.conf.username,
                    'os_password': self.conf.password,
                    'os_project_name': self.conf.project_name,
                    'os_auth_url': self.conf.auth_url,
                    'os_user_domain_id': self.conf.user_domain_id,
                    'os_project_domain_id': self.conf.project_domain_id,
                    'os_user_domain_name': self.conf.user_domain_name,
                    'os_project_domain_name': self.conf.project_domain_name,
                    'insecure': self.conf.disable_ssl_certificate_validation,
                    'cacert': self.conf.ca_file
                }
            }
        }

        zaqar = zaqarclient.Client(conf=conf)
        queue = zaqar.queue(queue_id)

        def validate_messages():
            messages = list(queue.messages())
            if len(messages) < 4:
                return False

            types = [m.body['type'] for m in messages]
            self.assertEqual(['os.heat.event'] * 4, types)
            resources = set([m.body['payload'][
                'resource_name'] for m in messages])
            self.assertEqual(set([stack_name, 'test_resource']), resources)
            stack_ids = [m.body['payload']['stack_id'] for m in messages]
            self.assertEqual([stack_id] * 4, stack_ids)
            statuses = [m.body['payload']['resource_status'] for m in messages]
            statuses.sort()
            self.assertEqual(['COMPLETE', 'COMPLETE',
                              'IN_PROGRESS', 'IN_PROGRESS'], statuses)
            actions = [m.body['payload']['resource_action'] for m in messages]
            self.assertEqual(['CREATE'] * 4, actions)
            return True

        self.assertTrue(test.call_until_true(20, 0, validate_messages))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import six

from heatclient import exc
from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class StackTemplateValidateTest(functional_base.FunctionalTestsBase):

    random_template = '''
heat_template_version: 2014-10-16
description: the stack description
parameters:
  aparam:
    type: number
    default: 10
    description: the param description
resources:
  myres:
    type: OS::Heat::RandomString
    properties:
      length: {get_param: aparam}
'''

    parent_template = '''
heat_template_version: 2014-10-16
description: the parent template
parameters:
  pparam:
    type: number
    default: 5
    description: the param description
resources:
  nres:
    type: mynested.yaml
    properties:
      aparam: {get_param: pparam}
'''

    parent_template_noprop = '''
heat_template_version: 2014-10-16
description: the parent template
resources:
  nres:
    type: mynested.yaml
'''

    random_template_groups = '''
heat_template_version: 2014-10-16
description: the stack description
parameters:
  aparam:
    type: number
    default: 10
    description: the param description
  bparam:
    type: string
    default: foo
  cparam:
    type: string
    default: secret
    hidden: true
parameter_groups:
- label: str_params
  description: The string params
  parameters:
  - bparam
  - cparam
resources:
  myres:
    type: OS::Heat::RandomString
    properties:
      length: {get_param: aparam}
'''

    @decorators.idempotent_id('b65a80c2-a507-4deb-9e7e-43181cc05211')
    def test_template_validate_basic(self):
        ret = self.client.stacks.validate(template=self.random_template)
        expected = {'Description': 'the stack description',
                    'Parameters': {
                        'aparam': {'Default': 10,
                                   'Description': 'the param description',
                                   'Label': 'aparam',
                                   'NoEcho': 'false',
                                   'Type': 'Number'}},
                    'Environment': {
                        'event_sinks': [],
                        'parameter_defaults': {},
                        'parameters': {},
                        'resource_registry': {u'resources': {}}}}
        self.assertEqual(expected, ret)

    @decorators.idempotent_id('bf27371d-e202-4bae-9f13-2ef137958517')
    def test_template_validate_override_default(self):
        env = {'parameters': {'aparam': 5}}
        ret = self.client.stacks.validate(template=self.random_template,
                                          environment=env)
        expected = {'Description': 'the stack description',
                    'Parameters': {
                        'aparam': {'Default': 10,
                                   'Value': 5,
                                   'Description': 'the param description',
                                   'Label': 'aparam',
                                   'NoEcho': 'false',
                                   'Type': 'Number'}},
                    'Environment': {
                        'event_sinks': [],
                        'parameter_defaults': {},
                        'parameters': {'aparam': 5},
                        'resource_registry': {u'resources': {}}}}
        self.assertEqual(expected, ret)

    @decorators.idempotent_id('0278e03d-ed50-4909-b29d-9c4267d3fcd6')
    def test_template_validate_override_none(self):
        env = {'resource_registry': {
               'OS::Heat::RandomString': 'OS::Heat::None'}}
        ret = self.client.stacks.validate(template=self.random_template,
                                          environment=env)
        expected = {'Description': 'the stack description',
                    'Parameters': {
                        'aparam': {'Default': 10,
                                   'Description': 'the param description',
                                   'Label': 'aparam',
                                   'NoEcho': 'false',
                                   'Type': 'Number'}},
                    'Environment': {
                        'event_sinks': [],
                        'parameter_defaults': {},
                        'parameters': {},
                        'resource_registry': {
                            'OS::Heat::RandomString': 'OS::Heat::None',
                            u'resources': {}}}}
        self.assertEqual(expected, ret)

    @decorators.idempotent_id('acb1435b-f1db-4427-9121-7e3144ddb81e')
    def test_template_validate_basic_required_param(self):
        tmpl = self.random_template.replace('default: 10', '')
        ret = self.client.stacks.validate(template=tmpl)
        expected = {'Description': 'the stack description',
                    'Parameters': {
                        'aparam': {'Description': 'the param description',
                                   'Label': 'aparam',
                                   'NoEcho': 'false',
                                   'Type': 'Number'}},
                    'Environment': {
                        'event_sinks': [],
                        'parameter_defaults': {},
                        'parameters': {},
                        'resource_registry': {u'resources': {}}}}
        self.assertEqual(expected, ret)

    @decorators.idempotent_id('7aac1feb-8256-4f70-8459-5e9780d28904')
    def test_template_validate_fail_version(self):
        fail_template = self.random_template.replace('2014-10-16', 'invalid')
        ex = self.assertRaises(exc.HTTPBadRequest,
                               self.client.stacks.validate,
                               template=fail_template)
        self.assertIn('The template version is invalid', six.text_type(ex))

    @decorators.idempotent_id('6a6472d2-71fa-4ebe-a2b6-20878838555b')
    def test_template_validate_parameter_groups(self):
        ret = self.client.stacks.validate(template=self.random_template_groups)
        expected = {'Description': 'the stack description',
                    'ParameterGroups':
                    [{'description': 'The string params',
                      'label': 'str_params',
                      'parameters': ['bparam', 'cparam']}],
                    'Parameters':
                    {'aparam':
                     {'Default': 10,
                      'Description': 'the param description',
                      'Label': 'aparam',
                      'NoEcho': 'false',
                      'Type': 'Number'},
                     'bparam':
                     {'Default': 'foo',
                      'Description': '',
                      'Label': 'bparam',
                      'NoEcho': 'false',
                      'Type': 'String'},
                     'cparam':
                     {'Default': 'secret',
                      'Description': '',
                      'Label': 'cparam',
                      'NoEcho': 'true',
                      'Type': 'String'}},
                    'Environment': {
                        'event_sinks': [],
                        'parameter_defaults': {},
                        'parameters': {},
                        'resource_registry': {u'resources': {}}}}
        self.assertEqual(expected, ret)

    @decorators.idempotent_id('5100cf18-f52a-47a2-880c-d540edad149f')
    def test_template_validate_nested_off(self):
        files = {'mynested.yaml': self.random_template}
        ret = self.client.stacks.validate(template=self.parent_template,
                                          files=files)
        expected = {'Description': 'the parent template',
                    'Parameters': {
                        'pparam': {'Default': 5,
                                   'Description': 'the param description',
                                   'Label': 'pparam',
                                   'NoEcho': 'false',
                                   'Type': 'Number'}},
                    'Environment': {
                        'event_sinks': [],
                        'parameter_defaults': {},
                        'parameters': {},
                        'resource_registry': {
                            u'mynested.yaml': u'mynested.yaml',
                            u'resources': {}}}}
        self.assertEqual(expected, ret)

    @decorators.idempotent_id('480bcf64-25ae-49c7-b147-7cbc27d09cea')
    def test_template_validate_nested_on(self):
        files = {'mynested.yaml': self.random_template}
        ret = self.client.stacks.validate(template=self.parent_template_noprop,
                                          files=files,
                                          show_nested=True)
        expected = {'Description': 'the parent template',
                    'Parameters': {},
                    'NestedParameters': {
                        'nres': {'Description': 'the stack description',
                                 'Parameters': {'aparam': {'Default': 10,
                                                           'Description':
                                                           'the param '
                                                           'description',
                                                           'Label': 'aparam',
                                                           'NoEcho': 'false',
                                                           'Type': 'Number'}},
                                 'Type': 'mynested.yaml'}},
                    'Environment': {
                        'event_sinks': [],
                        'parameter_defaults': {},
                        'parameters': {},
                        'resource_registry': {
                            u'mynested.yaml': u'mynested.yaml',
                            u'resources': {}}}}
        self.assertEqual(expected, ret)

    @decorators.idempotent_id('a0bb07f0-2e10-4226-a205-a7eb04df415f')
    def test_template_validate_nested_on_multiple(self):
        # parent_template -> nested_template -> random_template
        nested_template = self.random_template.replace(
            'OS::Heat::RandomString', 'mynested2.yaml')
        files = {'mynested.yaml': nested_template,
                 'mynested2.yaml': self.random_template}
        ret = self.client.stacks.validate(template=self.parent_template,
                                          files=files,
                                          show_nested=True)

        n_param2 = {'myres': {'Description': 'the stack description',
                              'Parameters': {'aparam': {'Default': 10,
                                                        'Description':
                                                        'the param '
                                                        'description',
                                                        'Label': 'aparam',
                                                        'NoEcho': 'false',
                                                        'Type': 'Number'}},
                              'Type': 'mynested2.yaml'}}
        expected = {'Description': 'the parent template',
                    'Parameters': {
                        'pparam': {'Default': 5,
                                   'Description': 'the param description',
                                   'Label': 'pparam',
                                   'NoEcho': 'false',
                                   'Type': 'Number'}},
                    'NestedParameters': {
                        'nres': {'Description': 'the stack description',
                                 'Parameters': {'aparam': {'Default': 10,
                                                           'Description':
                                                           'the param '
                                                           'description',
                                                           'Label': 'aparam',
                                                           'Value': 5,
                                                           'NoEcho': 'false',
                                                           'Type': 'Number'}},
                                 'NestedParameters': n_param2,
                                 'Type': 'mynested.yaml'}},
                    'Environment': {
                        'event_sinks': [],
                        'parameter_defaults': {},
                        'parameters': {},
                        'resource_registry': {
                            u'mynested.yaml': u'mynested.yaml',
                            'resources': {}}}}
        self.assertEqual(expected, ret)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from heatclient import exc
import six
from tempest.lib import decorators

from heat_tempest_plugin.common import test
from heat_tempest_plugin.tests.functional import functional_base


class RemoteStackTest(functional_base.FunctionalTestsBase):
    template = '''
heat_template_version: 2013-05-23
resources:
  my_stack:
    type: OS::Heat::Stack
    properties:
      context:
$MULTI_CLOUD_PROPERTIES
        region_name: RegionOne
      template:
        get_file: remote_stack.yaml
outputs:
  key:
    value: {get_attr: [my_stack, outputs]}
'''

    remote_template = '''
heat_template_version: 2013-05-23
resources:
  random1:
    type: OS::Heat::RandomString
outputs:
  remote_key:
    value: {get_attr: [random1, value]}
'''

    def setUp(self):
        super(RemoteStackTest, self).setUp()
        self.template = self.template.replace('$MULTI_CLOUD_PROPERTIES', '')
        # replacing the template region with the one from the config
        self.template = self.template.replace('RegionOne',
                                              self.conf.region)

    @decorators.idempotent_id('7e735c40-24fb-4ef8-8585-d1c68b344476')
    def test_remote_stack_alone(self):
        stack_id = self.stack_create(template=self.remote_template)
        expected_resources = {'random1': 'OS::Heat::RandomString'}
        self.assertEqual(expected_resources, self.list_resources(stack_id))
        stack = self.client.stacks.get(stack_id)
        output_value = self._stack_output(stack, 'remote_key')
        self.assertEqual(32, len(output_value))

    @decorators.idempotent_id('aeed20d1-ebda-4544-9ace-16b0c8fc24f6')
    def test_stack_create(self):
        files = {'remote_stack.yaml': self.remote_template}
        stack_id = self.stack_create(files=files)

        expected_resources = {'my_stack': 'OS::Heat::Stack'}
        self.assertEqual(expected_resources, self.list_resources(stack_id))

        stack = self.client.stacks.get(stack_id)
        output = self._stack_output(stack, 'key')
        parent_output_value = output['remote_key']
        self.assertEqual(32, len(parent_output_value))

        rsrc = self.client.resources.get(stack_id, 'my_stack')
        remote_id = rsrc.physical_resource_id
        rstack = self.client.stacks.get(remote_id)
        self.assertEqual(remote_id, rstack.id)
        remote_output_value = self._stack_output(rstack, 'remote_key')
        self.assertEqual(32, len(remote_output_value))
        self.assertEqual(parent_output_value, remote_output_value)

        remote_resources = {'random1': 'OS::Heat::RandomString'}
        self.assertEqual(remote_resources, self.list_resources(remote_id))

    def _create_with_cloud_credential(self):
        cred_sec_id = self.conf.credential_secret_id
        if not cred_sec_id:
            raise self.skipException(
                "No credential_secret_id configured to test")
        props = """
        credential_secret_id: %(credential_secret_id)s""" % {
            'credential_secret_id': cred_sec_id
        }

        self.template = self.template.replace('$MULTI_CLOUD_PROPERTIES', props)
        files = {'remote_stack.yaml': self.remote_template}
        stack_id = self.stack_create(files=files)

        expected_resources = {'my_stack': 'OS::Heat::Stack'}
        self.assertEqual(expected_resources, self.list_resources(stack_id))

        return stack_id

    @test.requires_feature('multi_cloud')
    @decorators.idempotent_id('6b61d8e3-79df-4e84-bdcf-f734da39d52b')
    def test_stack_create_with_cloud_credential(self):
        """Test on create multi (OpenStack) cloud with credential

        This test will use same region to simulate cross OpenStack scenario.
        Provide credential_secret_id as input property.
        """
        stack_id = self._create_with_cloud_credential()
        stack = self.client.stacks.get(stack_id)
        output = self._stack_output(stack, 'key')
        parent_output_value = output['remote_key']
        self.assertEqual(32, len(parent_output_value))

        rsrc = self.client.resources.get(stack_id, 'my_stack')
        remote_id = rsrc.physical_resource_id
        # For now we use same OpenStack environment as a simulation of remote
        # OpenStack site.
        rstack = self.client.stacks.get(remote_id)
        self.assertEqual(remote_id, rstack.id)
        remote_output_value = self._stack_output(rstack, 'remote_key')
        self.assertEqual(32, len(remote_output_value))
        self.assertEqual(parent_output_value, remote_output_value)

        remote_resources = {'random1': 'OS::Heat::RandomString'}
        self.assertEqual(remote_resources, self.list_resources(remote_id))

    @decorators.idempotent_id('830bfeae-6d8a-4cb2-823d-d8b6c3a740ad')
    def test_stack_create_bad_region(self):
        tmpl_bad_region = self.template.replace(self.conf.region, 'DARKHOLE')
        files = {'remote_stack.yaml': self.remote_template}
        kwargs = {
            'template': tmpl_bad_region,
            'files': files
        }
        ex = self.assertRaises(exc.HTTPBadRequest, self.stack_create, **kwargs)

        error_msg = ('ERROR: Cannot establish connection to Heat endpoint '
                     'at region "DARKHOLE" due to '
                     '"(?:public|internal|admin)(?:URL)? endpoint for '
                     'orchestration service in DARKHOLE region not found"')
        self.assertRegex(six.text_type(ex), error_msg)

    @decorators.idempotent_id('b2190dfc-d223-4595-b168-6c42b0f3a3e5')
    def test_stack_resource_validation_fail(self):
        tmpl_bad_format = self.remote_template.replace('resources', 'resource')
        files = {'remote_stack.yaml': tmpl_bad_format}
        kwargs = {'files': files}
        ex = self.assertRaises(exc.HTTPBadRequest, self.stack_create, **kwargs)

        error_msg = ('ERROR: Failed validating stack template using Heat '
                     'endpoint at region "%s" due to '
                     '"ERROR: The template section is '
                     'invalid: resource"') % self.conf.region
        self.assertEqual(error_msg, six.text_type(ex))

    @decorators.idempotent_id('141f0478-121b-4e61-bde7-d5551bfd1fc2')
    def test_stack_update(self):
        files = {'remote_stack.yaml': self.remote_template}
        stack_id = self.stack_create(files=files)

        expected_resources = {'my_stack': 'OS::Heat::Stack'}
        self.assertEqual(expected_resources, self.list_resources(stack_id))

        rsrc = self.client.resources.get(stack_id, 'my_stack')
        physical_resource_id = rsrc.physical_resource_id
        rstack = self.client.stacks.get(physical_resource_id)
        self.assertEqual(physical_resource_id, rstack.id)

        remote_resources = {'random1': 'OS::Heat::RandomString'}
        self.assertEqual(remote_resources,
                         self.list_resources(rstack.id))
        # do an update
        update_template = self.remote_template.replace('random1', 'random2')
        files = {'remote_stack.yaml': update_template}
        self.update_stack(stack_id, self.template, files=files)

        # check if the remote stack is still there with the same ID
        self.assertEqual(expected_resources, self.list_resources(stack_id))
        rsrc = self.client.resources.get(stack_id, 'my_stack')
        physical_resource_id = rsrc.physical_resource_id
        rstack = self.client.stacks.get(physical_resource_id)
        self.assertEqual(physical_resource_id, rstack.id)

        remote_resources = {'random2': 'OS::Heat::RandomString'}
        self.assertEqual(remote_resources,
                         self.list_resources(rstack.id))

    @decorators.idempotent_id('50841af8-bdf5-4df6-a075-dc061ada6833')
    def test_stack_suspend_resume(self):
        files = {'remote_stack.yaml': self.remote_template}
        stack_id = self.stack_create(files=files)
        self.stack_suspend(stack_id)
        self.stack_resume(stack_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from tempest.lib import decorators

from heat_tempest_plugin.common import exceptions
from heat_tempest_plugin.tests.scenario import scenario_base


class CfnInitIntegrationTest(scenario_base.ScenarioTestsBase):
    """Testing cfn-init and cfn-signal workability."""

    def setUp(self):
        super(CfnInitIntegrationTest, self).setUp()
        if not self.conf.image_ref:
            raise self.skipException("No image configured to test")
        if not self.conf.instance_type:
            raise self.skipException("No flavor configured to test")

    def check_stack(self, sid):
        # Check status of all resources
        for res in ('WaitHandle', 'SmokeSecurityGroup', 'SmokeKeys',
                    'CfnUser', 'SmokeServer', 'SmokeServerElasticIp'):
            self._wait_for_resource_status(
                sid, res, 'CREATE_COMPLETE')

        server_resource = self.client.resources.get(sid, 'SmokeServer')
        server_id = server_resource.physical_resource_id
        server = self.compute_client.servers.get(server_id)

        try:
            self._wait_for_resource_status(
                sid, 'WaitCondition', 'CREATE_COMPLETE')
        finally:
            # attempt to log the server console regardless of WaitCondition
            # going to complete. This allows successful and failed cloud-init
            # logs to be compared
            self._log_console_output(servers=[server])

        stack = self.client.stacks.get(sid)

        # This is an assert of great significance, as it means the following
        # has happened:
        # - cfn-init read the provided metadata and wrote out a file
        # - a user was created and credentials written to the server
        # - a cfn-signal was built which was signed with provided credentials
        # - the wait condition was fulfilled and the stack has changed state
        wait_status = json.loads(
            self._stack_output(stack, 'WaitConditionStatus'))
        self.assertEqual('smoke test complete', wait_status['smoke_status'])

        # Check EIP attributes.
        server_floatingip_id = self._stack_output(stack,
                                                  'ElasticIp_Id')
        self.assertIsNotNone(server_floatingip_id)

        # Fetch EIP details.
        net_show = self.network_client.show_floatingip(
            floatingip=server_floatingip_id)
        floating_ip = net_show['floatingip']['floating_ip_address']
        port_id = net_show['floatingip']['port_id']

        # Ensure that EIP was assigned to server.
        port_show = self.network_client.show_port(port=port_id)
        self.assertEqual(server.id, port_show['port']['device_id'])
        server_ip = self._stack_output(stack, 'SmokeServerElasticIp')
        self.assertEqual(server_ip, floating_ip)

        # Check that created server is reachable
        if not self._ping_ip_address(server_ip):
            self._log_console_output(servers=[server])
            self.fail(
                "Timed out waiting for %s to become reachable" % server_ip)

        # Check that the user can authenticate with the generated keypair
        if self.keypair:
            try:
                linux_client = self.get_remote_client(
                    server_ip, username='ec2-user')
                linux_client.validate_authentication()
            except (exceptions.ServerUnreachable,
                    exceptions.SSHTimeout):
                self._log_console_output(servers=[server])
                raise

    @decorators.idempotent_id('3f7726fc-a41b-40ca-ab38-51e2973f146a')
    def test_server_cfn_init(self):
        """Check cfn-init and cfn-signal availability on the created server.

        The alternative scenario is the following:
            1. Create a stack with a server and configured security group.
            2. Check that all stack resources were created.
            3. Check that created server is reachable.
            4. Check that stack was created successfully.
            5. Check that is it possible to connect to server
               via generated keypair.
        """
        parameters = {
            'key_name': self.keypair_name,
            'flavor': self.conf.instance_type,
            'image': self.conf.image_ref,
            'timeout': self.conf.build_timeout,
            'subnet': self.net['subnets'][0],
        }
        if self.conf.vm_to_heat_api_insecure:
            parameters['SignalExtraArgs'] = '--insecure'

        # Launch stack
        stack_id = self.launch_stack(
            template_name="test_server_cfn_init.yaml",
            parameters=parameters,
            expected_status=None
        )

        # Check stack
        self.check_stack(stack_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.lib import decorators

from heat_tempest_plugin.tests.functional import functional_base


class LoadBalancerv2Test(functional_base.FunctionalTestsBase):

    create_template = '''
heat_template_version: 2016-04-08
parameters:
    subnet:
        type: string
resources:
  loadbalancer:
    type: OS::Neutron::LBaaS::LoadBalancer
    properties:
      description: aLoadBalancer
      vip_subnet: { get_param: subnet }
  listener:
    type: OS::Neutron::LBaaS::Listener
    properties:
      description: aListener
      loadbalancer: { get_resource: loadbalancer }
      protocol: HTTP
      protocol_port: 80
      connection_limit: 5555
  pool:
    type: OS::Neutron::LBaaS::Pool
    properties:
      description: aPool
      lb_algorithm: ROUND_ROBIN
      protocol: HTTP
      listener: { get_resource: listener }
  poolmember:
    type: OS::Neutron::LBaaS::PoolMember
    properties:
      address: 1.1.1.1
      pool: { get_resource: pool }
      protocol_port: 1111
      subnet: { get_param: subnet }
      weight: 255
  # pm2
  healthmonitor:
    type: OS::Neutron::LBaaS::HealthMonitor
    properties:
      delay: 3
      type: HTTP
      timeout: 3
      max_retries: 3
      pool: { get_resource: pool }
outputs:
  loadbalancer:
    value: { get_attr: [ loadbalancer, show ] }
  pool:
    value: { get_attr: [ pool, show ] }
  poolmember:
    value: { get_attr: [ poolmember, show ] }
  listener:
    value: { get_attr: [ listener, show ] }
  healthmonitor:
    value: { get_attr: [ healthmonitor, show ] }
'''

    add_member = '''
  poolmember2:
    type: OS::Neutron::LBaaS::PoolMember
    properties:
      address: 2.2.2.2
      pool: { get_resource: pool }
      protocol_port: 2222
      subnet: { get_param: subnet }
      weight: 222
'''

    def setUp(self):
        super(LoadBalancerv2Test, self).setUp()
        if not self.is_network_extension_supported('lbaasv2'):
            self.skipTest('LBaasv2 extension not available, skipping')

    @decorators.idempotent_id('2f4a476c-cba7-448b-a7c1-85f7284f0293')
    def test_create_update_loadbalancer(self):
        parameters = {
            'subnet': self.conf.fixed_subnet_name,
        }
        stack_identifier = self.stack_create(template=self.create_template,
                                             parameters=parameters)
        stack = self.client.stacks.get(stack_identifier)
        output = self._stack_output(stack, 'loadbalancer')
        self.assertEqual('ONLINE', output['operating_status'])

        template = self.create_template.replace('ROUND_ROBIN', 'SOURCE_IP')
        template = template.replace('3', '6')
        template = template.replace('255', '256')
        template = template.replace('5555', '7777')
        template = template.replace('aLoadBalancer', 'updatedLoadBalancer')
        template = template.replace('aPool', 'updatedPool')
        template = template.replace('aListener', 'updatedListener')
        self.update_stack(stack_identifier, template=template,
                          parameters=parameters)
        stack = self.client.stacks.get(stack_identifier)

        output = self._stack_output(stack, 'loadbalancer')
        self.assertEqual('ONLINE', output['operating_status'])
        self.assertEqual('updatedLoadBalancer', output['description'])
        output = self._stack_output(stack, 'pool')
        self.assertEqual('SOURCE_IP', output['lb_algorithm'])
        self.assertEqual('updatedPool', output['description'])
        output = self._stack_output(stack, 'poolmember')
        self.assertEqual(256, output['weight'])
        output = self._stack_output(stack, 'healthmonitor')
        self.assertEqual(6, output['delay'])
        self.assertEqual(6, output['timeout'])
        self.assertEqual(6, output['max_retries'])
        output = self._stack_output(stack, 'listener')
        self.assertEqual(7777, output['connection_limit'])
        self.assertEqual('updatedListener', output['description'])

    @decorators.idempotent_id('104f59ae-a3c8-4c12-98e5-a7bc0007878d')
    def test_add_delete_poolmember(self):
        parameters = {
            'subnet': self.conf.fixed_subnet_name,
        }
        stack_identifier = self.stack_create(template=self.create_template,
                                             parameters=parameters)
        stack = self.client.stacks.get(stack_identifier)
        output = self._stack_output(stack, 'loadbalancer')
        self.assertEqual('ONLINE', output['operating_status'])
        output = self._stack_output(stack, 'pool')
        self.assertEqual(1, len(output['members']))
        # add pool member
        template = self.create_template.replace('# pm2', self.add_member)
        self.update_stack(stack_identifier, template=template,
                          parameters=parameters)
        stack = self.client.stacks.get(stack_identifier)
        output = self._stack_output(stack, 'loadbalancer')
        self.assertEqual('ONLINE', output['operating_status'])
        output = self._stack_output(stack, 'pool')
        self.assertEqual(2, len(output['members']))
        # delete pool member
        self.update_stack(stack_identifier, template=self.create_template,
                          parameters=parameters)
        stack = self.client.stacks.get(stack_identifier)
        output = self._stack_output(stack, 'loadbalancer')
        self.assertEqual('ONLINE', output['operating_status'])
        output = self._stack_output(stack, 'pool')
        self.assertEqual(1, len(output['members']))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

import os

default_template = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    'tests/scenario/templates/boot_config_none_env.yaml')

service_available_group = cfg.OptGroup(name="service_available",
                                       title="Available OpenStack Services")

ServiceAvailableGroup = [
    cfg.BoolOpt("heat",
                default=True,
                help="Whether or not heat is expected to be available",
                deprecated_opts=[cfg.DeprecatedOpt(
                    'heat_plugin',
                    group='service_available')]),
]

heat_group = cfg.OptGroup(name="heat_plugin",
                          title="Heat Service Options")

HeatGroup = [
    cfg.StrOpt("catalog_type",
               default="orchestration",
               help="Catalog type of the orchestration service."),
    cfg.StrOpt('username',
               help="Username to use for non admin API requests."),
    cfg.StrOpt('password',
               help="Non admin API key to use when authenticating.",
               secret=True),
    cfg.StrOpt('admin_username',
               help="Username to use for admin API requests."),
    cfg.StrOpt('admin_password',
               help="Admin API key to use when authentication.",
               secret=True),
    cfg.StrOpt('project_name',
               help="Project name to use for API requests.",
               deprecated_opts=[cfg.DeprecatedOpt(
                   'tenant_name', group='heat_plugin')]),
    cfg.StrOpt('admin_project_name',
               default='admin',
               help="Admin project name to use for admin API requests.",
               deprecated_opts=[cfg.DeprecatedOpt(
                   'admin_tenant_name', group='heat_plugin')]),
    cfg.StrOpt('auth_url',
               help="Full URI of the OpenStack Identity API (Keystone)."),
    cfg.StrOpt('auth_version',
               help="OpenStack Identity API version."),
    cfg.StrOpt('user_domain_name',
               help="User domain name, if keystone v3 auth_url "
                    "is used"),
    cfg.StrOpt('project_domain_name',
               help="Project domain name, if keystone v3 auth_url "
                    "is used"),
    cfg.StrOpt('user_domain_id',
               help="User domain id, if keystone v3 auth_url "
                    "is used"),
    cfg.StrOpt('project_domain_id',
               help="Project domain id, if keystone v3 auth_url "
                    "is used"),
    cfg.StrOpt('region',
               help="The region name to use"),
    cfg.StrOpt('endpoint_type',
               default='public',
               choices=['public', 'admin', 'internal'],
               help="The endpoint type to use for the orchestration service."),
    cfg.StrOpt('instance_type',
               help="Instance type for tests. Needs to be big enough for a "
                    "full OS plus the test workload"),
    cfg.StrOpt('minimal_instance_type',
               help="Instance type enough for simplest cases."),
    cfg.StrOpt('image_ref',
               help="Name of image to use for tests which boot servers."),
    cfg.StrOpt('keypair_name',
               help="Name of existing keypair to launch servers with."),
    cfg.StrOpt('minimal_image_ref',
               help="Name of minimal (e.g cirros) image to use when "
                    "launching test instances."),
    cfg.BoolOpt('disable_ssl_certificate_validation',
                default=False,
                help="Set to True if using self-signed SSL certificates."),
    cfg.StrOpt('ca_file',
               help="CA certificate to pass for servers that have "
                    "https endpoint."),
    cfg.IntOpt('build_interval',
               default=4,
               help="Time in seconds between build status checks."),
    cfg.IntOpt('build_timeout',
               default=1200,
               help="Timeout in seconds to wait for a stack to build."),
    cfg.StrOpt('network_for_ssh',
               default='heat-net',
               help="Network used for SSH connections."),
    cfg.StrOpt('fixed_network_name',
               default='heat-net',
               help="Visible fixed network name "),
    cfg.StrOpt('floating_network_name',
               default='public',
               help="Visible floating network name "),
    cfg.StrOpt('boot_config_env',
               default=default_template,
               help="Path to environment file which defines the "
                    "resource type Heat::InstallConfigAgent. Needs to "
                    "be appropriate for the image_ref."),
    cfg.StrOpt('fixed_subnet_name',
               default='heat-subnet',
               help="Visible fixed sub-network name "),
    cfg.IntOpt('ssh_timeout',
               default=300,
               help="Timeout in seconds to wait for authentication to "
                    "succeed."),
    cfg.IntOpt('ip_version_for_ssh',
               default=4,
               help="IP version used for SSH connections."),
    cfg.IntOpt('ssh_channel_timeout',
               default=60,
               help="Timeout in seconds to wait for output from ssh "
                    "channel."),
    cfg.BoolOpt('skip_scenario_tests',
                default=False,
                help="Skip all scenario tests"),
    cfg.BoolOpt('skip_functional_tests',
                default=False,
                help="Skip all functional tests"),
    cfg.ListOpt('skip_functional_test_list',
                help="List of functional test class or class.method "
                     "names to skip ex. AutoscalingGroupTest, "
                     "InstanceGroupBasicTest.test_size_updates_work"),
    cfg.ListOpt('skip_scenario_test_list',
                help="List of scenario test class or class.method "
                     "names to skip ex. NeutronLoadBalancerTest, "
                     "AodhAlarmTest.test_alarm"),
    cfg.ListOpt('skip_test_stack_action_list',
                help="List of stack actions in tests to skip "
                     "ex. ABANDON, ADOPT, SUSPEND, RESUME"),
    cfg.BoolOpt('convergence_engine_enabled',
                default=True,
                help="Test features that are only present for stacks with "
                     "convergence enabled."),
    cfg.IntOpt('volume_size',
               default=1,
               help='Default size in GB for volumes created by volumes tests'),
    cfg.IntOpt('connectivity_timeout',
               default=120,
               help="Timeout in seconds to wait for connectivity to "
                    "server."),
    cfg.StrOpt('heat_config_notify_script',
               default=('heat-config-notify'),
               help="Path to the script heat-config-notify"),
    cfg.StrOpt('hidden_stack_tag',
               default='data-processing-cluster',
               help="Tag to be considered as hidden for stack tags tests"),
    cfg.StrOpt('credential_secret_id',
               help="Barbican secret id which storing cloud credential in "
               "remote site."),
    cfg.BoolOpt('vm_to_heat_api_insecure',
                default=False,
                help="Set this to True if VM images used for tests "
                     "can not verify a (self-signed) SSL certificate "
                     "of public Heat endpoint."),
]

heat_features_group = cfg.OptGroup(
    name='heat_features_enabled',
    title="Enabled Orchestration Service Features")

HeatFeaturesGroup = [
    cfg.BoolOpt('stack_cancel',
                default=False,
                help="If false, skip stack cancel tests"),
    cfg.BoolOpt('multi_cloud',
                default=False,
                help="If false, skip multi-cloud tests for remote stack")
]


def list_opts():
    yield heat_group.name, HeatGroup
    yield heat_features_group.name, HeatFeaturesGroup
    yield service_available_group.name, ServiceAvailableGroup
